/**
 * Copyright 2007 ZeroTurnaround.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

package org.zeroturnaround.javarebel;

import java.net.URLClassLoader;

/**
 * <p>Provides hooks to integrate a custom class loader with JavaRebel.</p>
 * 
 * <p>A typical integration will look like this:</p>
 * 
 * <pre>
 * class MyClassLoader extends URLClassLoader {
 *   public MyClassLoader() {
 *     ...
 *     IntegrationFactory.getInstance().registerUrlClassLoader(this);
 *   }
 *   ...
 *   public Class findClass(String classname) {
 *      Class result = 
 *        IntegrationFactory.getInstance()
 *          .findReloadableClass(this, classname);
 *      if (result != null)
 *        return result;
 *      ...
 *   }   
 * }
 * </pre>
 * 
 * <p>Note that when using custom integration on Java 5 the <code>-javaagent</code> should still be enabled.</p>
 * 
 * @author Jevgeni Kabanov (ekabanov@zeroturnaround.com)
 */
public interface Integration {
  /**
   * Registers a <code>ClassLoader</code> with JavaRebel associating with it
   * the class-to-file resolution strategy provided by {@link ClassFileSource}. It
   * is necessary to do this before any calls to {@link #findReloadableClass(ClassLoader, String)},
   * the best place is usually the constructor.
   * 
   * @param cl The custom class loader being integrated.
   * @param cfs The class-to-file resolution strategy source. 
   * 
   * @see ClassFileSource
   */
  public void registerClassLoader(ClassLoader cl, ClassResourceSource cfs);  
  
  /**
   * Registers an <code>URLClassLoader</code> or its descendant with JavaRebel.
   * Uses a default class-to-file resolution strategy, which deduces the file 
   * from {@link URLClassLoader#findResource(String)} result URL. Will fail if the 
   * class loader does not use "file://" protocol for URLs.
   *  
   * @param cl The custom class loader being integrated.
   * 
   * @see #registerClassLoader(ClassLoader, ClassResourceSource)
   */
  public void registerUrlClassLoader(URLClassLoader cl);
    
  /**
   * Resolves classes managed by JavaRebel. These and only these classes
   * will be reloadable. Returns <b>null</b> if class couldn't be resolved.
   * 
   * @param cl The custom class loader being integrated.
   * @param classname The public class name.
   * @return Resolved class or <b>null</b>.
   */
  public Class findReloadableClass(ClassLoader cl, String classname) throws ClassNotFoundException;
}