package org.zeroturnaround.javarebel.support;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * 
 * @author ekabanov
 *
 */
public class JarResourceCache {
  private static final Map zipFileCache = new HashMap();
  
  private static Long lock = new Long(1);

  /**
   * 
   * @param url
   * @return
   */
  public static JarClassResource get(URL url) {
    ClassLoader cl = JarResourceCache.class.getClassLoader();
    if (cl == null)
      cl = ClassLoader.getSystemClassLoader();

    synchronized (cl) {
      synchronized (lock) {
        try {      
          String[] split = url.getPath().split("!/");
          
          String zipFileName = split[0];
          
          try {
            zipFileName = new URL(zipFileName).getPath();
          } 
          catch (MalformedURLException e) {
            // OK
          }
          
          File zipFile = new File(zipFileName);
          String entryName = split[1];
          JarCache zCache = (JarCache) zipFileCache.get(zipFile.getCanonicalPath());

          if (zCache == null) {
            zCache = new JarCache(zipFile);
            zipFileCache.put(zipFile.getCanonicalPath(), zCache);
          }

          return new JarClassResource(zCache, url, entryName);
        } 
        catch (IOException e) {
          throw new RuntimeException(e); 
        }
      }
    }

  }

  /**
   * 
   * @author ekabanov
   *
   */
  static class JarCache {
    private static final int BUFFER_SIZE = 8194;

    private File file;
    private long previousFileLastModified = Long.MIN_VALUE;
    private Map entryLastModifiedCache = new HashMap();
    private boolean manifestReloadable = false;

    public JarCache(File file) {
      this.file = file;        
      previousFileLastModified = 0;
    }

    private void resetCacheIfNecessary() {
      if (file.lastModified() <= previousFileLastModified)
        return;

      JarFile jf;
      try {
        jf = new JarFile(file);
      } 
      catch (IOException e1) {
        throw new RuntimeException(e1);
      };

      try {        
        Manifest mf = jf.getManifest();        
        manifestReloadable = 
          mf != null 
          && mf.getMainAttributes() != null 
          && mf.getMainAttributes().getValue("JavaRebel-Reloadable") != null;       


        for (Enumeration i = jf.entries(); i.hasMoreElements();) {
          JarEntry entry = (JarEntry) i.nextElement();

          if (entry.getTime() != -1)
            entryLastModifiedCache.put(entry.getName(), new Long(entry.getTime()));         
        }
      } 
      catch (IOException e) {
        throw new RuntimeException(e);
      }
      finally {
        try {
          jf.close();
        } 
        catch (IOException e) {
          e.printStackTrace();
        }
      }

      previousFileLastModified = file.lastModified();
    }
    
    public synchronized byte[] getEntryBytes(String entryName) {
      try {
        // Create a ZipInputStream to read the zip file
        FileInputStream fis = new FileInputStream(file);
        ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));

        try {
          // Loop over all of the entries in the zip file
          int count;
          byte data[] = new byte[BUFFER_SIZE];
          ZipEntry entry;

          while ((entry = zis.getNextEntry()) != null) {
            if (entry.getName().equals(entryName)) {
              // Write the file to the file system
              ByteArrayOutputStream fos = new ByteArrayOutputStream(BUFFER_SIZE);
              while ((count = zis.read(data, 0, BUFFER_SIZE)) != -1) {
                fos.write(data, 0, count);
              }
              fos.flush();
              fos.close();

              return fos.toByteArray();
            }
          }

          throw new RuntimeException("Entry '" + entryName + "' in zip file '" + file.getName() + "' does not exist anymore!");
        } 
        finally {
          fis.close();
          zis.close();
        }
      } 
      catch (IOException e) {
        throw new RuntimeException(e);
      }
    }    

    /**
     * 
     * @param entryName
     * @return
     */
    public synchronized long getEntryLastModified(String entryName) {
      resetCacheIfNecessary();

      Long lm = (Long) entryLastModifiedCache.get(entryName);
      if (lm == null)
        return file.lastModified();

      return lm.longValue();

    }

    public synchronized boolean isManifestReloadable() {
      resetCacheIfNecessary();

      return manifestReloadable;
    }

    public File getFile() {
      return file;
    }
  }
}
